	TITLE	<WORD PROCESS FILTER FOR ASSEMBLERS>
	COM	<'WPFLT - a word processing printer >
	COM	<'filter   Ver 2.0.1    06/03/86 '>
	COM	<'Bill McQueen'>
;
;
;This filter is for use with LDOS 5.1.x and printer listings
;of assembly language source code.  During the code portion
;of a line, i.e. before the semi-colon, this filter has no
;effect on the character stream sent to the printer.
;However, between the semi-colon and the carriage return
;(i.e. the comment section of the line) the filter switches
;into a "word processing" mode.  During this mode, any time
;AN "ESCAPE CHARACTER" IS SENT TO THE PRINTER, THE FILTER
;causes the first character following the "escape character"
;to be interpreted as a control code and toggles certain
;printer modes (e.g. underlining) ON/OFF.  This allows
;the writer to enhance the comments by highlighting important
;information.
;
;The "escape symbol" used in this listing is the "@". This
;may be changed as desired by changing the defination for
;ESC_SYM in the EQUATES list.
;
;The following word processor functions are supported by
;this program as written (Others may be added as desired):
;
;	@-	Toggles underlining On/Off
;	@+	Toggles Bold Face ON/OFF
;	@/	Toggles Italics ON/OFF
;	@@	Prints the "@" character
;
;--------------------------------------------------------
;
ESC_SYM	EQU	'@'		;Escape Symbol to be used
				;in text to be printed. May
				;be changed as desired.
LF	EQU	10
CR	EQU	13
ESC	EQU	27
PRPORT	EQU	0F8H		;Model III printer port
@EXIT	EQU	402DH
@ABORT	EQU	4030H
HIGH$	EQU	4411H
@DSPLY	EQU	4467H
@LOGOT	EQU	428AH
@WHERE	EQU	000BH
;==========================================================
;	The following code is the relocator and is discarded
;	after the filter is is place and activated.  The
;	program loads at 5200H and the filter proper is
;	relocated to high memory just below the current
;	HIGH$, then HIGH$ is modified to protect the
;	filter. A standard header is put in place with 
;	the filter so that it is recoginized by high
;	memory mapping programs.
;
;
	ORG	5200H
ENTRY	LD	A,(DE)		;get device type
	AND	2		;make sure it is an
	JR	Z,NOGOOD	;output device.
	PUSH	DE		;save device DCB.
	LD	HL,MSG		;point to initialization
	CALL	@DSPLY		;mesage and display it.
	POP	IX		;recover device DCB.
	LD	HL,(HIGH$)	;reduce HIGH$ by the
	LD	(STORE),HL
	LD	BC,LAST-START	;length of this driver.
	XOR	A		;clear the carry flag.
	SBC	HL,BC		;calculate new HIGH$.
	LD	(HIGH$),HL	;driver now protected.
	INC	HL		;point HL at new start.
	LD	A,(IX+1)	;xfer orig DCB vector
	LD	(PUTBXT+1),A	;to driver CALL.
	LD	(GETBYT+1),A
	LD	A,(IX+2)
	LD	(PUTBXT+2),A
	LD	(GETBYT+2),A
;---------------
	PUSH	HL
	XOR	A		;clear carry
	LD	DE,START
	SBC	HL,DE		;hl = new start - old
	XOR	A		;clear carry
	LD	DE,WP_CTL
	ADC	HL,DE		;hl = new WP_CTL
	LD	(MARK1+2),HL
	POP	HL
;-------------------------------
	DI			;not during update.
	LD	(IX+1),L		;update DCB Vector
	LD	(IX+2),H		;to filter entry.
	EX	DE,HL		;xfer new START to DE.
	LD	HL,START	;load address of driver.
	LDIR			;move driver to top.
	EI			;enable interrupts again.
	JP	@EXIT		;return to LDOS READY.
;
;******************************************************
;
;	Error Handling Routine
;
NOGOOD	LD	HL,ERR_MSG
	CALL	@DSPLY
	JP	@ABORT		;abort to LDOS.
MSG	DB	LF,LF,'          Installing'
	DM	LF,'    Word Processing Filter'
	DM	LF,'             for'
	DM	LF,'  Assembler Printer Listings'
	DM	LF,'         LDOS  5.1.x'
	DM	LF,'             and'
	DM	LF,'         EPSON MX-80',LF
	DM	LF,'   written by:  Bill McQueen'
	DM	LF,'       Date:    06/03/86'
	DM	LF,'          Ver:  2.0.1'
	DM	CR
ERR_MSG	DM	LF,'***ERROR*** --- For output only.'
	DM	LF,CR
;
;****************************************************
;
;	The following code is the actual filter
;	which is moved to HIGH$.
;
START	JR	TSTART		;Standard himem HEADER
STORE	DEFW	0		;stg for old HIGH$ value
NAME	DB	TSTART-TEXT
TEXT	DB	'WPFLT 2.0.1'
TSTART	JR	C,GETBYT	;don't filter a GET rqst.
;
	PUSH	BC		;save registers.
	PUSH	DE
	PUSH	HL
	PUSH	IX
	PUSH	IY
	PUSH	AF
	JR	STRT_WP	;go to word processing
;
;
PUTBYT	EQU	$		;exit to original driver.
	POP	AF		;restore registers.
	POP	IY
	POP	IX
	POP	HL
	POP	DE
	POP	BC
PUTBXT	JP	0
;
GETBYT	JP	0		;to original driver
;
DMPBYT	EQU	$	;routine to return without
			;sending original character to
			;printer.
	POP	AF		;restore registers.
	POP	IY
	POP	IX
	POP	HL
	POP	DE
	POP	BC
	LD	A,C
	RET
;
;
;******************************************************
;
WP_CTL	EQU	$		;Word Processing
				;Routine Control Block.
;##############################################################
;To make word processing mode effective at all times change
;this line to "DB    1", and delete the five lines of code 
;indicated above.
;
	DB	0		;1 = WP mode , 0 = normal
;
;###############################################################
	DB	0		;1= ESC mode, 0= normal
	DB	0		;0FH=Underline on, 0=off
	DB	0		;0FH= Italics on, 0= off
	DB	0		;0Fh= Bold prt on, 0= off
;
;
;_________________________________________________________
STRT_WP	EQU	$
MARK1	LD	IX,WP_CTL	;IX-> Routine Cntl Block
	LD	A,(IX+1)	;Fetch Escape mode flag.
	CP	1		;Are we in Excape Mode??
	JR	Z,ESCAPE	;If so handle it.
;########################################################
;
;To make wp filter effective at all times for possible use
;(with basic print lines etc), delete the following five
;lines of code and change the first "DB	  0" in the WP_CTL
;(Word Processing Control Block) to "DB   1"
;
	LD	A,C		;A = char enroute to prtr.
	CP	';'		;Start Comments???
	JR	Z,SET_WP	;Enter Word Proc mode.
	CP	0DH		;End-of-line???
	JR	Z,RESET		;End WP Mode.
;
;########################################################
	LD	A,(IX)		;Fetch WP mode
	CP	0
	JR	Z,PUTBYT	;skip if not WP mode
	LD	A,C		;A = char enroute to prtr
	CP	ESC_SYM		;Escape Symbol???
	JR	Z,SET_ESC	;Enter ESCAPE Mode.
	JR	PUTBYT
;
;------------------------------------------------------
;
TO_PUTBYT2	JR	PUTBYT
TO_DMPBYT2	JR	DMPBYT
;
;-------------------------------------------------------
;
;The following two routines enable or disable the word
;processing facilities as a semi-colon or carriage
;return respectively pass thru the filter and then
;send character on for printing.
;
SET_WP	EQU	$
	LD	(IX),01H	;Flag WP Mode ON.
	JR	PUTBYT
;
RESET	EQU	$
	LD	(IX),0		;Flag WP Mode OFF.
	LD	(IX+1),0	;Reset all Feature flags
	LD	(IX+2),0
	LD	(IX+3),0
	LD	(IX+4),0
	LD	A,ESC		;Reset all Feature modes
	CALL	@WHERE		;in printer.
	JR	BYTOUT
	LD	A,2DH		;'-'
	CALL	@WHERE
	JR	BYTOUT
	LD	A,0		;UL off.
	CALL	@WHERE
	JR	BYTOUT
	LD	A,ESC
	CALL	@WHERE
	JR	BYTOUT
	LD	A,35H		;Italics off
	CALL	@WHERE
	JR	BYTOUT
	LD	A,ESC
	CALL	@WHERE
	JR	BYTOUT
	LD	A,48H		;Bold off
	CALL	@WHERE
	JR	BYTOUT
	JR	TO_PUTBYT
;
;------------------------------------------------------
;
TO_PUTBYT	JR	TO_PUTBYT2
TO_DMPBYT	JR	TO_DMPBYT2
;
;-------------------------------------------------------
;
;The following routine sets the ESCAPE mode and then
;returns to EDAS without printing any character.
;
SET_ESC	LD	(IX+1),1	;Escape mode on.
	JR	TO_DMPBYT	;return w/o printing.
;
;-----------------------------------------------------
;
;ESCAPE DECODER.
;
ESCAPE	EQU	$
	LD	(IX+1),0	;reset ESCAPE mode.
	LD	A,C		;A = character
	CP	'-'
	JR	Z,UNDERLN		;Toggle Underlining.
	CP	'/'
	JR	Z,ITALICS		;Toggle Italics.
	CP	'+'
	JR	Z,BOLD		;Toggle Bold printing.
	JR	TO_PUTBYT		;Otherwise print char.
;
;-----------------------------------------------------
;
BYTOUT	EQU	$
	INC	HL		;set-up for return
	INC	HL
	PUSH	HL
	PUSH	BC
	LD	B,A		;save byte
LOOP1	IN	A,(PRPORT)	;fetch printer status
	AND	0F0H		;mask
	CP	30H
	JR	NZ,LOOP1	;wait if not ready
	LD	A,B		;restore A
	POP	BC		;restore B
	OUT	(PRPORT),A	;send byte
	RET
;
;--------------------------------------------------
;
UNDERLN	EQU	$
	LD	A,(IX+2)	;Fetch UL status.
	CPL			;Toggle it.
	LD	(IX+2),A	;Store new status.
	PUSH	AF		;save register
	LD	A,ESC		;A = Escape Character.
	CALL	@WHERE		;Send it to printer.
	JR	BYTOUT
	LD	A,2DH		;'-'
	CALL	@WHERE
	JR	BYTOUT
	POP	AF		;restore register
	CALL	@WHERE		;Send to printer.
	JR	BYTOUT
	JR	TO_DMPBYT
;
ITALICS	EQU	$
	LD	A,(IX+3)	;Fetch Italics status.
	CPL			;Toggle it.
	LD	(IX+3),A	;STore new status.
	LD	A,ESC		;A = Escape Char
	CALL	@WHERE		;Send to printer.
	JR	BYTOUT
	LD	A,35H		;Make Italic code.
	ADD	A,(IX+3)
	CALL	@WHERE		;Send to printer.
	JR	BYTOUT
	JR	TO_DMPBYT
;
BOLD	EQU	$
	LD	A,(IX+4)	;Fetch Bold Status.
	CPL			;Toggle it.
	LD	(IX+4),A	;Store new status.
	LD	A,ESC		;A = ESCAPE character
	CALL	@WHERE		;Send it to printer
	JR	BYTOUT
	LD	A,48H		;Make printer code.
	ADD	A,(IX+4)
	CALL	@WHERE		;Send to printer
	JR	BYTOUT
	JR	TO_DMPBYT
;
;__________________________________________________________
;
;******************************************************
LAST	EQU	$
	END	ENTRY
